#---------------------------------------------------------------------------
#
#   MWG - Editing screen
#
#---------------------------------------------------------------------------

from __future__ import division
from pygame import draw
from pygame.transform import rotozoom
import pygame.locals as pk
from albow.widget import Widget
from albow.resource import get_font
from albow.layout import Row, Column
from albow.controls import Label
from albow.fields import TextEditor
from albow.dialogs import Dialog
from albow.palette_view import PaletteView
from albow.resource import get_image
from humerus.edit_commands import EditCommandScreen
from level_view import LevelView
from constants import *
from utils import make_yarn
import tiles
from moving_objects import MovingObject, YarnBall
from vector_algebra import vadd, vsub

#---------------------------------------------------------------------------

TOOL_SELECT = 0
TOOL_YARN = 1
TOOL_GROUND = 2
TOOL_YARNBALL = 3
TOOL_MYNA = 4
TOOL_HOLE = 5
TOOL_OBSTACLE = 6
TOOL_PAVING = 7
TOOL_MESSAGE = 8
TOOL_NOROLL = 9
TOOL_ITEM = 10

tool_images = {
	TOOL_SELECT: get_image("tools/select.png"),
	TOOL_YARN: get_image("tools/yarn.png"),
	TOOL_GROUND: get_image("tools/ground.png"),
	TOOL_YARNBALL: get_image("tools/yarnball.png"),
	TOOL_MYNA: get_image("tools/myna.png"),
	TOOL_HOLE: get_image("tools/hole.png"),
	TOOL_OBSTACLE: get_image("tools/obstacle.png"),
	TOOL_PAVING: get_image("tools/paving.png"),
	TOOL_MESSAGE: get_image("tools/message.png"),
	TOOL_NOROLL: get_image("tools/noroll.png"),
	TOOL_ITEM: get_image("tools/item.png"),
}

num_tools = len(tool_images)

yarn_zone_to_dir = {
	(1, 0): DIR_NORTH,
	(2, 1): DIR_EAST,
	(1, 2): DIR_SOUTH,
	(0, 1): DIR_WEST,
}

no_roll_overlay = get_image("no_roll_overlay.png")

current_tool = TOOL_SELECT
#current_tool = TOOL_YARN
#current_tool = TOOL_GROUND

#---------------------------------------------------------------------------

class MwgEditScreen(EditCommandScreen):

	def __init__(self, shell):
		EditCommandScreen.__init__(self, shell)
		self.game = shell.game
		level_view = EditingLevelView(self.game)
		tool_palette = ToolPalette()
		tile_menu = TileMenu(level_view)
		toolbar1 = Row([MenuButton("Tiles", tile_menu)])
		toolbar2 = Row([tool_palette])
		toolbars = Column([toolbar1, toolbar2], spacing = 4)
		toolbars.bottomleft = (0, self.height)
		self.add(level_view)
		self.add(toolbars)
		level_view.focus()

#---------------------------------------------------------------------------

class EditingLevelView(LevelView):

	grid_color = (255, 255, 255)
	selection_color = (0, 0, 255)
	hole_color = (255, 0, 0)

	show_cell_grid = True
	current_tile_index = 1 #0

	def __init__(self, game):
		LevelView.__init__(self, game)
		self.game = game
		self.selection = []
	
	def level(self):
		return self.game.level

	def clear_selection(self):
		del self.selection[:]
	
	def add_to_selection(self, obj):
		self.selection.append(obj)
	
	def remove_from_selection(self, obj):
		self.selection.remove(obj)
	
	def object_is_selected(self, obj):
		return obj in self.selection
	
	def selected_moving_objects(self):
		for obj in self.selection:
			if isinstance(obj, MovingObject):
				yield obj
	
	def post_draw_object(self, surf, obj):
		if self.object_is_selected(obj):
			r = self.get_moving_object_rect(obj)
			draw.rect(surf, self.selection_color, r, 1)

	def post_draw_cell(self, surf, cell, r):
		if cell.no_roll:
			self.draw_overlay(surf, cell, r, no_roll_overlay)
		if cell.is_hole:
			draw.ellipse(surf, self.hole_color, r, 5)
	
	def draw_overlay(self, surf, cell, frame, image):
		r = image.get_rect()
		r.center = frame.center
		surf.blit(image, r)

	def mouse_down(self, e):
		tool = current_tool
		if tool == TOOL_SELECT:
			self.select_click(e)
		elif tool == TOOL_YARN:
			self.yarn_click(e)
		elif tool == TOOL_GROUND:
			self.ground_click(e)
		elif tool == TOOL_YARNBALL:
			self.yarnball_click(e)
		elif tool == TOOL_MYNA:
			self.myna_click(e)
		elif tool == TOOL_HOLE:
			self.hole_click(e)
		elif tool == TOOL_OBSTACLE:
			self.obstacle_click(e)
		elif tool == TOOL_PAVING:
			self.paving_click(e)
		elif tool == TOOL_MESSAGE:
			self.message_click(e)
		elif tool == TOOL_NOROLL:
			self.noroll_click(e)
		elif tool == TOOL_ITEM:
			self.item_click(e)
	
	def mouse_drag(self, e):
		tool = current_tool
		if tool == TOOL_SELECT:
			self.select_drag(e)
		elif tool == TOOL_YARN:
			self.yarn_drag(e)
	
	def mouse_up(self, e):
		tool = current_tool
		if tool == TOOL_YARN:
			self.yarn_release(e)
	
	def select_click(self, e):
		if not e.shift:
			self.clear_selection()
		objects = self.level().moving_objects
		for i in xrange(len(objects) - 1, -1, -1):
			obj = objects[i]
			r = self.get_moving_object_rect(obj)
			if r.collidepoint(e.local):
				if e.shift and self.object_is_selected(obj):
					self.remove_from_selection(obj)
				else:
					self.add_to_selection(obj)
		self.drag_pt = self.view_to_level_point(e.local)
	
	def select_drag(self, e):
		p0 = self.drag_pt
		p1 = self.view_to_level_point(e.local)
		v = vsub(p1, p0)
		for obj in self.selected_moving_objects():
			obj.position = vadd(obj.position, v)
			self.level_changed()
		self.drag_pt = p1
	
	def yarn_click(self, e):
		self.yarn_point = self.find_yarn_point(e.local)
	
	def yarn_drag(self, e):
		yp0 = self.yarn_point
		if yp0:
			yp1 = self.find_yarn_point(e.local)
			if yp1 and yp0 != yp1:
				cell = yp0[0]
				if cell is yp1[0]:
					yarn = make_yarn(yp0[1], yp1[1], None)
					cell.toggle_yarn(yarn)
					self.level_changed()
				self.yarn_point = yp1
	
	def yarn_release(self, e):
		yp = self.yarn_point
		self.yarn_point = None
		if yp:
			dir = yp[1]
			zone = self.yarn_zone(e.local)
			if zone == (1, 1):
				coords = self.view_to_cell_coords(e.local)
				for obj in self.level().moving_objects_in_cell(coords):
					if isinstance(obj, YarnBall):	
						if obj.yarn_dir == dir:
							obj.yarn_dir = None
						else:
							obj.yarn_dir = dir
						break
	
	def find_yarn_point(self, p):
		cell = self.find_cell(p)
		if cell:
			dir = self.yarn_dir(p)
			if dir is not None:
				return (cell, dir)
	
	def ground_click(self, e):
		cell = self.find_cell(e.local)
		if cell:
			cell.ground = tiles.tile_name_with_index(self.current_tile_index)
			self.level_changed()
	
	def yarnball_click(self, e):
		p = self.view_to_level_point(e.local)
		obj = YarnBall()
		obj.position = p
		self.level().add_moving_object(obj)
		self.level_changed()
	
	def myna_click(self, e):
		p = self.view_to_level_point(e.local)
		self.level().player.position = p
		self.level_changed()
	
	def hole_click(self, e):
		cell = self.find_cell(e.local)
		if cell:
			cell.is_hole = not cell.is_hole
			self.level_changed()
	
	def obstacle_click(self, e):
		self.toggle_tile(e, 'obstacle')
	
	def paving_click(self, e):
		self.toggle_tile(e, 'pavement')
	
	def message_click(self, e):
		cell = self.find_cell(e.local)
		if cell:
			self.edit_cell_message(cell)
	
	def noroll_click(self, e):
		cell = self.find_cell(e.local)
		if cell:
			cell.no_roll = not cell.no_roll
			self.level_changed()
	
	def item_click(self, e):
		self.toggle_tile(e, 'item')
	
	def key_down(self, e):
		k = e.key
		if k == pk.K_RIGHT:
			self.move_window(1, 0)
			return
		elif k == pk.K_DOWN:
			self.move_window(0, 1)
			return
		elif k == pk.K_LEFT:
			self.move_window(-1, 0)
			return
		elif k == pk.K_UP:
			self.move_window(0, -1)
			return
		elif e.cmd and e.alt:
			if k == pk.K_w:
				self.expand_level(1, 0)
				return
			elif k == pk.K_h:
				self.expand_level(0, 1)
				return
		LevelView.key_down(self, e)
	
	def toggle_tile(self, e, attr):
		cell = self.find_cell(e.local)
		if cell:
			if e.shift:
				setattr(cell, attr, None)
			else:
				setattr(cell, attr, tiles.tile_name_with_index(self.current_tile_index))
			self.level_changed()

	def view_to_level_point(self, p):
		i0, j0 = self.get_window_pos()
		cw, ch = self.cell_size
		x = i0 + p[0] / cw
		y = j0 + p[1] / ch
		return (x, y)
	
	def view_to_cell_coords(self, p):
		x, y = self.view_to_level_point(p)
		return int(x), int(y)
	
	def yarn_dir(self, p):
		zone = self.yarn_zone(p)
		return yarn_zone_to_dir.get(zone)
		
	def yarn_zone(self, p):
		cw, ch = self.cell_size
		x = (3 * (p[0] % cw)) // cw
		y = (3 * (p[1] % ch)) // ch
		return (x, y)

	def level_changed(self):
		self.game.level_changed()
		self.invalidate()
	
	def edit_cell_message(self, cell):
		dlog = Dialog()
		field = TextEditor(width = 780)
		field.topleft = (4, 4)
		#field.enter_action = dlog.ok
		#field.escape_action = dlog.cancel
		field.text = cell.message
		dlog.add(field)
		dlog.shrink_wrap()
		if dlog.present():
			cell.message = field.text
			self.level_changed()
	
	def expand_level(self, sw, sh):
		ww, wh = self.window_size
		self.level().change_size(sw * (ww - 1), sh * (wh - 1))
		self.level_changed()
	
	def move_window(self, dx, dy):
		p = vadd(self.get_window_pos(), (dx, dy))
		self.move_window_to(p)
	
#	def move_window(self, dx, dy):
#		lw, lh = self.level().level_size
#		ww, wh = self.window_size
#		wx, wy = vadd(self.get_window_pos(), (dx, dy))
#		wx = max(0, min(wx, lw - ww))
#		wy = max(0, min(wy, lh - wh))
#		#print "Level: moving window to", (wx, wy) ###
#		self.set_window_pos((wx, wy))

	def adjust_window(self):
		pass

#---------------------------------------------------------------------------

class ToolPalette(PaletteView):

	def __init__(self):
		PaletteView.__init__(self, (32, 32), 1, num_tools)
	
	def num_items(self):
		return num_tools
	
	def draw_item(self, surf, i, rect):
		image = tool_images[i]
		r = image.get_rect()
		r.center = rect.center
		surf.blit(image, r)
	
	def click_item(self, i, e):
		global current_tool
		current_tool = i
	
	def item_is_selected(self, i):
		return current_tool == i

#---------------------------------------------------------------------------

class MenuButton(Label):

	font = get_font(15, "VeraBd.ttf")

	def __init__(self, text, menu):
		Label.__init__(self, text)
		menu.topleft = (4, 4)
		self.menu = menu
	
	def mouse_down(self, e):
		dlog = Dialog(border_width = 1)
		dlog.add(self.menu)
		dlog.shrink_wrap()
		dlog.present()

#---------------------------------------------------------------------------

class TileMenu(PaletteView):
	
	def __init__(self, client):
		PaletteView.__init__(self, (42, 32), 16, 16, scrolling = True)
		self.client = client
	
	def num_items(self):
		return len(tiles.tile_list)
	
	def draw_item(self, surf, i, rect):
		image = rotozoom(tiles.tile_image_with_index(i), 0.0, 0.5)
		r = image.get_rect()
		r.center = rect.center
		surf.blit(image, r)
	
	def click_item(self, i, e):
		self.client.current_tile_index = i
		self.parent.dismiss()
	
	def item_is_selected(self, i):
		return self.client.current_tile_index == i
