#---------------------------------------------------------------------------
#
#   MWG - Level
#
#---------------------------------------------------------------------------

from math import ceil
import moving_objects
from constants import *
from utils import make_yarn, yarns_conflict
from behaviour import obstacle_info

#---------------------------------------------------------------------------

class Level(object):

	level_size = (10, 9)  # cells wide, cells high
	window_pos = (0, 0)    # top left cell visible in window
	
	def __init__(self):
		w, h = self.level_size
		self.cells = [
			[Cell() for i in xrange(h)]
				for j in xrange(w)]
		self.moving_objects = []
		self.player = moving_objects.Player()
		self.add_moving_object(self.player)

	def add_test_objects(self):
		yarnball = moving_objects.YarnBall()
		self.add_moving_object(yarnball)
		self.cells[1][1].yarn.append(YARN_EW)
		self.cells[2][1].yarn.append(YARN_SW)
		self.cells[2][2].yarn.extend([YARN_NS, YARN_EW])
		self.cells[2][3].yarn.append(YARN_NS)
		self.cells[4][4].yarn.append(YARN_NS)
		yarnball.yarn_dir = DIR_NORTH
	
	def change_size(self, ncols, nrows):
		w, h = self.level_size
		cells = self.cells
		for i in xrange(w):
			cells[i].extend(make_cells(nrows))
		for i in xrange(ncols):
			cells.append(make_cells(h + nrows))
		self.level_size = (w + ncols, h + nrows)
		print "Level: changed size to", self.level_size ###
		print "...num cols =", len(self.cells) ###
	
	def cell_at(self, coords):
		i, j = coords
		return self.cells[i][j]
	
	def safe_cell_at(self, coords):
		i, j = coords
		lw, lh = self.level_size
		if 0 <= i < lw and 0 <= j <= lh:
			return self.cells[i][j]
	
	def add_moving_object(self, obj):
		self.moving_objects.append(obj)
		obj.level = self
	
	def remove_moving_object(self, obj):
		try:
			self.moving_objects.remove(obj)
		except ValueError:
			pass
	
	def reset(self):
		#print "Level.reset" ###
		self.is_completed = False
		for col in self.cells:
			for cell in col:
				cell.reset()
		for obj in self.moving_objects:
			obj.reset()
	
	def moving_objects_in_row(self, j):
		result = []
		for obj in self.moving_objects:
			if int(obj.position[1]) == j:
				result.append(obj)
		return result
	
	def moving_objects_in_cell(self, coords):
		result = []
		for obj in self.moving_objects:
			if obj.is_in_cell(coords):
				result.append(obj)
		return result
	
	def begin_frame(self):
		for obj in self.moving_objects:
			obj.begin_frame()

	def try_to_move_object(self, obj, dir, dist):
		x, y = obj.position
		old_cell_coords = (int(x), int(y))
		r = obj.radius
		#print "Level.try_to_move_object:", dir, dist ###
		#print "...orig pos =", (x, y) ###
		#print "...radius =", r ###
		lw, lh = self.level_size
		def hit_cells(x1, y1, x2, y2):
			cells = self.cells
			for i in xrange(max(0, int(x1)), min(lw, int(ceil(x2)))):
				for j in xrange(max(0, int(y1)), min(lh, int(ceil(y2)))):
					cell = cells[i][j]
					if cell.blocked_by_obstacle():
						if not obj.try_to_remove_obstacle(cell):
							return True
			return False
		if dir == DIR_EAST:
			x += dist
			x2 = x + r
			h = x2 > self.level_size[0] or hit_cells(x2, y - r, x2, y + r)
		elif dir == DIR_SOUTH:
			y += dist
			y2 = y + r
			h = y2 > self.level_size[1] or hit_cells(x - r, y2, x + r, y2)
		elif dir == DIR_WEST:
			x -= dist
			x2 = x - r
			h = x - r < 0 or hit_cells(x2, y - r, x2, y + r)
		else:
			y -= dist
			y2 = y - r
			h = y2 < 0 or hit_cells(x - r, y2, x + r, y2)
		#print "...new pos =", (x, y) ###
		#print "...hit level bounds =", h ###
		if h:
			return False
		new_cell_coords = (int(x), int(y))
		if new_cell_coords != old_cell_coords:
			if not obj.can_move_into_cell(dir, self.cell_at(new_cell_coords)):
				return False
		for obj2 in self.moving_objects:
			if obj2.hit_by_movement(obj, dir, dist):
				#print "...hit by movement:", obj2 ###
				if not obj2.pushable or \
					not self.try_to_move_object(obj2, dir, dist):
						return False
		obj.change_position(dir, (x, y))
		return True
	
	def key_down(self, e):
		self.player.key_down(e)

	def key_up(self, e):
		self.player.key_up(e)

	def finish(self):
		self.is_completed = True
	
	def cell_containing_player(self):
		coords = self.player.cell_coords()
		return self.cell_at(coords)

#---------------------------------------------------------------------------

class Cell(object):

	ground = "grass"
	pavement = None
	obstacle = None
	is_hole = False
	message = ""
	no_roll = False
	item = None
	
	object_falling_in = None

	def __init__(self):
		self.yarn = []

	def add_yarn(self, dir1, dir2, owner):
		self.yarn.append(make_yarn(dir1, dir2, owner))
	
	def remove_yarn(self, dir, owner):
		j, i = self.find_yarn(dir, owner)
		if j is not None:
			yarns = self.yarn
			yarn = yarns[j]
			del yarns[j]
			return yarn[1 - i]

	def yarn_submerged(self, dir, owner):
		j, i = self.find_yarn(dir, owner)
		if j is not None:
			yarns = self.yarn
			yarn = yarns[j]
			for i in xrange(j + 1, len(yarns)):
				if yarns_conflict(yarn, yarns[i]):
					return True
		return False

	def find_yarn(self, dir, owner):
		yarns = self.yarn
		for j in xrange(len(yarns) - 1, -1, -1):
			yarn = yarns[j]
			if yarn[2] is owner:
				for i in (0, 1):
					if yarn[i] == dir:
						return j, i
		return None, None
	
	def set_yarn_owner(self, j, owner):
		yarns = self.yarn
		yarn = yarns[j]
		yarns[j] = make_yarn(yarn[0], yarn[1], owner)

	def toggle_yarn(self, new_yarn):
		print "Cell.toggle_yarn:", new_yarn ###
		print "...before:", self.yarn ###
		for j, old_yarn in enumerate(self.yarn):
			if old_yarn[:2] == new_yarn[:2]:
				del self.yarn[j]
				break
		else:
			self.yarn.append(new_yarn)
		print "...after:", self.yarn ###

	def blocked_by_obstacle(self):
		obstacle = self.obstacle
		if not obstacle:
			return False
		info = obstacle_info(obstacle)
		return not info or not info.open
	
	def reset(self):
		yarns = self.yarn
		for j, yarn in enumerate(yarns):
			if len(yarn) < 3:
				yarns[j] = make_yarn(yarn[0], yarn[1], None)
				#print "Cell.reset: changed yarn to", yarns[j] ###

#---------------------------------------------------------------------------

def make_cells(n):
	return [Cell() for i in xrange(n)]

